@@ -24,13 +24,13 @@
  */
 
 #include <common.h>
-#include <div64.h>
 #include <asm/io.h>
 #include <asm/errno.h>
 #include <asm/arch/imx-regs.h>
 #include <asm/arch/crm_regs.h>
 #include <asm/arch/clock.h>
 #include <div64.h>
+#include <asm/arch/sys_proto.h>
 
 enum pll_clocks {
 	PLL1_CLOCK = 0,
@@ -60,6 +60,9 @@ struct mxc_pll_reg *mxc_plls[PLL_CLOCKS] = {
 #define EMI_DIV_MAX     8
 #define NFC_DIV_MAX     8
 
+#define MX5_CBCMR	0x00015154
+#define MX5_CBCDR	0x02888945
+
 struct fixed_pll_mfd {
 	u32 ref_clk_hz;
 	u32 mfd;
@@ -226,7 +229,7 @@ u32 get_mcu_main_clk(void)
 /*
  * Get the rate of peripheral's root clock.
  */
-static u32 get_periph_clk(void)
+u32 get_periph_clk(void)
 {
 	u32 reg;
 
@@ -246,15 +249,6 @@ static u32 get_periph_clk(void)
 	/* NOTREACHED */
 }
 
-static u32 get_ahb_clk(void)
-{
-	u32 cbcdr = __raw_readl(&mxc_ccm->cbcdr);
-	u32 pdf = (cbcdr & MXC_CCM_CBCDR_AHB_PODF_MASK) \
-			>> MXC_CCM_CBCDR_AHB_PODF_OFFSET;
-
-	return  get_periph_clk() / (pdf + 1);
-}
-
 /*
  * Get the rate of ipg clock.
  */
@@ -349,7 +343,7 @@ static u32 get_lp_apm(void)
 /*
  * get cspi clock rate.
  */
-static u32 get_cspi_clk(void)
+static u32 imx_get_cspiclk(void)
 {
 	u32 ret_val = 0, pdf, pre_pdf, clk_sel;
 	u32 cscmr1 = __raw_readl(&mxc_ccm->cscmr1);
@@ -417,15 +411,6 @@ static u32 get_emi_slow_clk(void)
 	return  get_periph_clk() / (pdf + 1);
 }
 
-static u32 get_nfc_clk(void)
-{
-	u32 cbcdr = __raw_readl(&mxc_ccm->cbcdr);
-	u32 pdf = (cbcdr & MXC_CCM_CBCDR_NFC_PODF_MASK) \
-			>> MXC_CCM_CBCDR_NFC_PODF_OFFSET;
-
-	return  get_emi_slow_clk() / (pdf + 1);
-}
-
 static u32 get_ddr_clk(void)
 {
 	u32 ret_val = 0;
@@ -464,9 +449,8 @@ static u32 get_ddr_clk(void)
 	return ret_val;
 }
 
-
 /*
- * The API of get mxc clockes.
+ * The API of get mxc clocks.
  */
 unsigned int mxc_get_clock(enum mxc_clock clk)
 {
@@ -482,16 +466,18 @@ unsigned int mxc_get_clock(enum mxc_clock clk)
 	case MXC_UART_CLK:
 		return get_uart_clk();
 	case MXC_CSPI_CLK:
-		return get_cspi_clk();
+		return imx_get_cspiclk();
 	case MXC_FEC_CLK:
 		return decode_pll(mxc_plls[PLL1_CLOCK],
 				    CONFIG_SYS_MX5_HCLK);
+	case MXC_SATA_CLK:
+		return get_ahb_clk();
 	case MXC_DDR_CLK:
 		return get_ddr_clk();
 	default:
 		break;
 	}
-	return -1;
+	return -EINVAL;
 }
 
 u32 imx_get_uartclk(void)
@@ -505,11 +491,6 @@ u32 imx_get_fecclk(void)
 	return mxc_get_clock(MXC_IPG_CLK);
 }
 
-/*
- * Clock config code start here
- */
-
-/* precondition: m>0 and n>0.  Let g=gcd(m,n). */
 static int gcd(int m, int n)
 {
 	int t;
@@ -529,12 +510,6 @@ static int gcd(int m, int n)
  * targeted clock based on the equation:
  *      t_clk = 2*ref_freq*(mfi + mfn/(mfd+1))/(pd+1)
  * This calculation is based on a fixed MFD value for simplicity.
- *
- * @param ref       reference clock freq in Hz
- * @param target    targeted clock in Hz
- * @param pll       pll_param structure.
- *
- * @return          0 if successful; non-zero otherwise.
  */
 static int calc_pll_params(u32 ref, u32 target, struct pll_param *pll)
 {
@@ -552,7 +527,7 @@ static int calc_pll_params(u32 ref, u32 target, struct pll_param *pll)
 			"within [%d - %d]\n",
 			PLL_FREQ_MIN(ref) / SZ_DEC_1M,
 			PLL_FREQ_MAX(ref) / SZ_DEC_1M);
-		return -1;
+		return -EINVAL;
 	}
 
 	for (i = 0; i < ARRAY_SIZE(fixed_mfd); i++) {
@@ -563,7 +538,7 @@ static int calc_pll_params(u32 ref, u32 target, struct pll_param *pll)
 	}
 
 	if (i == ARRAY_SIZE(fixed_mfd))
-		return -1;
+		return -EINVAL;
 
 	/* Use n_target and n_ref to avoid overflow */
 	for (pd = 1; pd <= PLL_PD_MAX; pd++) {
@@ -571,25 +546,24 @@ static int calc_pll_params(u32 ref, u32 target, struct pll_param *pll)
 		do_div(t1, (4 * n_ref));
 		mfi = t1;
 		if (mfi > PLL_MFI_MAX)
-			return -1;
+			return -EINVAL;
 		else if (mfi < 5)
 			continue;
 		break;
 	}
-	/* Now got pd and mfi already */
 	/*
-	mfn = (((n_target * pd) / 4 - n_ref * mfi) * mfd) / n_ref;
-	*/
+	 * Now got pd and mfi already
+	 *
+	 * mfn = (((n_target * pd) / 4 - n_ref * mfi) * mfd) / n_ref;
+	 */
 	t1 = n_target * pd;
 	do_div(t1, 4);
 	t1 -= n_ref * mfi;
 	t1 *= mfd;
 	do_div(t1, n_ref);
 	mfn = t1;
-#ifdef CMD_CLOCK_DEBUG
-	printf("ref=%d, target=%d, pd=%d," "mfi=%d,mfn=%d, mfd=%d\n",
+	debug("ref=%d, target=%d, pd=%d," "mfi=%d,mfn=%d, mfd=%d\n",
 		ref, n_target, (u32)pd, (u32)mfi, (u32)mfn, (u32)mfd);
-#endif
 	i = 1;
 	if (mfn != 0)
 		i = gcd(mfd, mfn);
@@ -614,43 +588,6 @@ static int calc_pll_params(u32 ref, u32 target, struct pll_param *pll)
 		(v - 1);				\
 	})
 
-static u32 calc_per_cbcdr_val(u32 per_clk)
-{
-	u32 cbcdr = __raw_readl(&mxc_ccm->cbcdr);
-	u32 tmp_clk = 0, div = 0, clk_sel = 0;
-
-	cbcdr &= ~MXC_CCM_CBCDR_PERIPH_CLK_SEL;
-
-	/* emi_slow_podf divider */
-	tmp_clk = get_emi_slow_clk();
-	clk_sel = cbcdr & MXC_CCM_CBCDR_EMI_CLK_SEL;
-	if (clk_sel) {
-		div = calc_div(tmp_clk, per_clk, 8);
-		cbcdr &= ~MXC_CCM_CBCDR_EMI_PODF_MASK;
-		cbcdr |= (div << MXC_CCM_CBCDR_EMI_PODF_OFFSET);
-	}
-
-	/* axi_b_podf divider */
-	tmp_clk = get_axi_b_clk();
-	div = calc_div(tmp_clk, per_clk, 8);
-	cbcdr &= ~MXC_CCM_CBCDR_AXI_B_PODF_MASK;
-	cbcdr |= (div << MXC_CCM_CBCDR_AXI_B_PODF_OFFSET);
-
-	/* axi_b_podf divider */
-	tmp_clk = get_axi_a_clk();
-	div = calc_div(tmp_clk, per_clk, 8);
-	cbcdr &= ~MXC_CCM_CBCDR_AXI_A_PODF_MASK;
-	cbcdr |= (div << MXC_CCM_CBCDR_AXI_A_PODF_OFFSET);
-
-	/* ahb podf divider */
-	tmp_clk = AHB_CLK_ROOT;
-	div = calc_div(tmp_clk, per_clk, 8);
-	cbcdr &= ~MXC_CCM_CBCDR_AHB_PODF_MASK;
-	cbcdr |= (div << MXC_CCM_CBCDR_AHB_PODF_OFFSET);
-
-	return cbcdr;
-}
-
 #define CHANGE_PLL_SETTINGS(pll, pd, fi, fn, fd) \
 	{	\
 		__raw_writel(0x1232, &pll->ctrl);		\
@@ -711,7 +648,7 @@ static int config_pll_clk(enum pll_clocks index, struct pll_param *pll_param)
 		__raw_writel(ccsr & ~0x20, &mxc_ccm->ccsr);
 		break;
 	default:
-		return -1;
+		return -EINVAL;
 	}
 
 	return 0;
@@ -742,7 +679,7 @@ static int config_nfc_clk(u32 nfc_clk)
 	u32 div = parent_rate / nfc_clk;
 
 	if (nfc_clk <= 0)
-		return -1;
+		return -EINVAL;
 	if (div == 0)
 		div++;
 	if (parent_rate / div > NFC_CLK_MAX)
@@ -781,39 +718,8 @@ static int config_periph_clk(u32 ref, u32 freq)
 			return config_pll_clk(PLL3_CLOCK, &pll_param);
 			break;
 		default:
-			return -1;
+			return -EINVAL;
 		}
-	} else {
-		u32 old_cbcmr = __raw_readl(&mxc_ccm->cbcmr);
-		u32 new_cbcdr = calc_per_cbcdr_val(freq);
-		u32 old_nfc = get_nfc_clk();
-
-		/* Switch peripheral to PLL3 */
-		__raw_writel(0x00015154, &mxc_ccm->cbcmr);
-		__raw_writel(0x02888945, &mxc_ccm->cbcdr);
-
-		/* Make sure change is effective */
-		while (__raw_readl(&mxc_ccm->cdhipr) != 0)
-			;
-
-		/* Setup PLL2 */
-		ret = calc_pll_params(ref, freq, &pll_param);
-		if (ret != 0) {
-			printf("Error:Can't find pll parameters: %d\n",
-				ret);
-			return ret;
-		}
-		config_pll_clk(PLL2_CLOCK, &pll_param);
-
-		/* Switch peripheral back */
-		__raw_writel(new_cbcdr, &mxc_ccm->cbcdr);
-		__raw_writel(old_cbcmr, &mxc_ccm->cbcmr);
-
-		/* Make sure change is effective */
-		while (__raw_readl(&mxc_ccm->cdhipr) != 0)
-			;
-		/* restore to old NFC clock */
-		config_nfc_clk(old_nfc);
 	}
 
 	return 0;
@@ -849,7 +755,7 @@ static int config_ddr_clk(u32 emi_clk)
 		shift = 10;
 		break;
 	default:
-		return -1;
+		return -EINVAL;
 	}
 
 	if ((clk_src % emi_clk) < 10000000)
@@ -869,7 +775,7 @@ static int config_ddr_clk(u32 emi_clk)
 	return 0;
 }
 
-/*!
+/*
  * This function assumes the expected core clock has to be changed by
  * modifying the PLL. This is NOT true always but for most of the times,
  * it is. So it assumes the PLL output freq is the same as the expected
@@ -888,11 +794,6 @@ static int config_ddr_clk(u32 emi_clk)
  *      4) This function should not have allowed diag_printf() calls since
  *         the serial driver has been stoped. But leave then here to allow
  *         easy debugging by NOT calling the cyg_hal_plf_serial_stop().
- *
- * @param ref   pll input reference clock (24MHz)
- * @param freq  core clock in Hz
- * @param clk   clock type, e.g CPU_CLK, DDR_CLK, etc.
- * @return      0 if successful; non-zero otherwise
  */
 int mxc_set_clock(u32 ref, u32 freq, enum mxc_clock clk)
 {
@@ -901,19 +802,19 @@ int mxc_set_clock(u32 ref, u32 freq, enum mxc_clock clk)
 	switch (clk) {
 	case MXC_ARM_CLK:
 		if (config_core_clk(ref, freq))
-			return -1;
+			return -EINVAL;
 		break;
 	case MXC_PERIPH_CLK:
 		if (config_periph_clk(ref, freq))
-			return -1;
+			return -EINVAL;
 		break;
 	case MXC_DDR_CLK:
 		if (config_ddr_clk(freq))
-			return -1;
+			return -EINVAL;
 		break;
 	case MXC_NFC_CLK:
 		if (config_nfc_clk(freq))
-			return -1;
+			return -EINVAL;
 		break;
 	default:
 		printf("Warning:Unsupported or invalid clock type\n");
@@ -922,6 +823,29 @@ int mxc_set_clock(u32 ref, u32 freq, enum mxc_clock clk)
 	return 0;
 }
 
+#ifdef CONFIG_MX53
+/*
+ * The clock for the external interface can be set to use internal clock
+ * if fuse bank 4, row 3, bit 2 is set.
+ * This is an undocumented feature and it was confirmed by Freescale's support:
+ * Fuses (but not pins) may be used to configure SATA clocks.
+ * Particularly the i.MX53 Fuse_Map contains the next information
+ * about configuring SATA clocks :  SATA_ALT_REF_CLK[1:0] (offset 0x180C)
+ * '00' - 100MHz (External)
+ * '01' - 50MHz (External)
+ * '10' - 120MHz, internal (USB PHY)
+ * '11' - Reserved
+*/
+void mxc_set_sata_internal_clock(void)
+{
+	u32 *tmp_base =
+		(u32 *)(IIM_BASE_ADDR + 0x180c);
+
+	set_usb_phy1_clk();
+
+	writel((readl(tmp_base) & (~0x6)) | 0x4, tmp_base);
+}
+#endif
 
 /*
  * Dump some core clockes.
@@ -940,11 +864,13 @@ int do_mx5_showclocks(cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
 	freq = decode_pll(mxc_plls[PLL4_CLOCK], CONFIG_SYS_MX5_HCLK);
 	printf("PLL4       %8d MHz\n", freq / 1000000);
 #endif
+
 	printf("\n");
 	printf("AHB        %8d kHz\n", mxc_get_clock(MXC_AHB_CLK) / 1000);
 	printf("IPG        %8d kHz\n", mxc_get_clock(MXC_IPG_CLK) / 1000);
 	printf("IPG PERCLK %8d kHz\n", mxc_get_clock(MXC_IPG_PERCLK) / 1000);
 	printf("DDR        %8d kHz\n", mxc_get_clock(MXC_DDR_CLK) / 1000);
+
 	return 0;
 }
 
