@@ -31,16 +31,30 @@
 /*
  * High Level Configuration Options
  */
-#define CONFIG_PANDA		1	/* working with Panda */
+#define CONFIG_PANDA	/* working with Panda */
 
-/* USB Host networking options */
-#define CONFIG_CMD_USB			1
-#define CONFIG_USB_STORAGE		1
-#define CONFIG_USB_HOST_ETHER		1
-#define CONFIG_USB_EHCI			1
-#define CONFIG_USB_EHCI_OMAP4		1
-#define CONFIG_USB_HOST			1
-#define CONFIG_USB_ETHER_SMSC95XX	1
+/* USB UHH support options */
+#define CONFIG_CMD_USB
+#define CONFIG_USB_HOST
+#define CONFIG_USB_EHCI
+#define CONFIG_USB_EHCI_OMAP
+#define CONFIG_USB_STORAGE
+#define CONFIG_SYS_USB_EHCI_MAX_ROOT_PORTS 3
+
+#define CONFIG_OMAP_EHCI_PHY1_RESET_GPIO 1
+#define CONFIG_OMAP_EHCI_PHY2_RESET_GPIO 62
+
+/* USB Networking options */
+#define CONFIG_USB_HOST_ETHER
+#define CONFIG_USB_ETHER_SMSC95XX
+
+#define CONFIG_UBOOT_ENABLE_PADS_ALL
+
+#define CONFIG_CMD_PING
+#define CONFIG_CMD_DHCP
+
+#define CONFIG_USB_ULPI
+#define CONFIG_USB_ULPI_VIEWPORT_OMAP
 
 /* BOOTP options */
 #define CONFIG_BOOTP_BOOTFILESIZE
@@ -54,12 +68,8 @@
 #define CONFIG_BOOTP_PXE
 #define CONFIG_BOOTP_PXE_CLIENTARCH	0x100
 #define CONFIG_BOOTP_VCI_STRING		"U-boot.armv7.omap4_panda"
-
-
-/* Ethernet */
-#define CONFIG_NET_MULTI		1
-#define CONFIG_CMD_PING			1
-#define CONFIG_CMD_DHCP			1
+#define CONFIG_BOOTP_VENDOREX
+#define CONFIG_BOOTP_VENDOREX_PXE_SHARED
 
 #include <configs/omap4_common.h>
 #define CONFIG_CMD_NET
